/*
 * Copyright (C) 2008-2012 The QXmpp developers
 *
 * Author:
 *  Manjeet Dahiya
 *
 * Source:
 *  http://code.google.com/p/qxmpp
 *
 * This file is a part of QXmpp library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 */


#ifndef QXMPPVCARDIQ_H
#define QXMPPVCARDIQ_H

#include "QXmppIq.h"
#include <QDate>
#include <QMap>
#include <QDomElement>

class QImage;

struct QXmppVCardPhone
{
    QXmppVCardPhone();

    bool isHome;
    bool isWork;
    bool isVoice;
    bool isFax;
    bool isPager;
    bool isMsg;
    bool isCell;
    bool isVideo;
    bool isBBS;
    bool isModem;
    bool isISDN;
    bool isPCS;
    bool isPref;

    QString number;
};
typedef QList<QXmppVCardPhone> QXmppVCardPhoneList;

struct QXmppVCardEmail
{
    QXmppVCardEmail();

    bool isHome;
    bool isWork;
    bool isInternet;
    bool isPref;
    bool isX400;

    QString address;
};
typedef QList<QXmppVCardEmail> QXmppVCardEmailList;

struct QXmppVCardAddress
{
    QXmppVCardAddress();

    bool isEmpty() const;

    bool isHome;
    bool isWork;
    bool isPostal;
    bool isPref;

    QString street;
    QString locality;
    QString region;
    QString pCode;
    QString country;
    QString extAdd;
};
typedef QList<QXmppVCardAddress> QXmppVCardAddressList;

/// \brief Represents the XMPP vCard.
///
/// The functions names are self explanatory.
/// Look at QXmppVCardManager and XEP-0054: vcard-temp for more details.
///
/// There are many field of XMPP vCard which are not present in
/// this class. File a issue for the same. We will add the requested
/// field to this class.
///

class QXMPP_EXPORT QXmppVCardIq : public QXmppIq
{
public:
    QXmppVCardIq(const QString& bareJid = "");

    QDate birthday() const;
    void setBirthday(const QDate &birthday);

    QString email() const;
    void setEmail(const QString&);

    QString firstName() const;
    void setFirstName(const QString&);

    QString fullName() const;
    void setFullName(const QString&);

    QString lastName() const;
    void setLastName(const QString&);

    QString middleName() const;
    void setMiddleName(const QString&);

    QString nickName() const;
    void setNickName(const QString&);

    QByteArray photo() const;
    void setPhoto(const QByteArray&);

    QString photoType() const;
    void setPhotoType(const QString &type);

    QString url() const;
    void setUrl(const QString&);

    QString orgName() const;
    void setOrgName(const QString&);

    QString orgUnit() const;
    void setOrgUnit(const QString&);

    QString title() const;
    void setTitle(const QString&);

    QString role() const;
    void setRole(const QString&);

    QString desc() const;
    void setDesc(const QString&);

    QXmppVCardPhoneList phones() const;
    void setPhones(const QXmppVCardPhoneList&);

    QXmppVCardEmailList emails() const;
    void setEmails(const QXmppVCardEmailList&);

    QXmppVCardAddressList addresses() const;
    void setAddresses(const QXmppVCardAddressList&);

    /// \cond
    static bool isVCard(const QDomElement &element);
    /// \endcond

protected:
    /// \cond
    void parseElementFromChild(const QDomElement&);
    void toXmlElementFromChild(QXmlStreamWriter *writer) const;
    /// \endcond

private:
    QDate m_birthday;
    QString m_email;
    QString m_firstName;
    QString m_fullName;
    QString m_lastName;
    QString m_middleName;
    QString m_nickName;
    QString m_url;
    QString m_orgName;
    QString m_orgUnit;
    QString m_title;
    QString m_role;
    QString m_desc;

    QXmppVCardPhoneList m_phones;
    QXmppVCardEmailList m_emails;
    QXmppVCardAddressList m_addresses;

    // not as 64 base
    QByteArray m_photo;
    QString m_photoType;
};

#endif // QXMPPVCARDIQ_H
